/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.installation.install.sl;

import java.net.URL;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.jbi.descriptor.original.generated.ClassLoaderDelegationType;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi.SharedLibrary;
import org.ow2.petals.microkernel.api.container.ContainerService;
import org.ow2.petals.microkernel.api.container.SharedLibraryLifeCycle;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.jbi.management.task.AbstractLoggableTask;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * This task create the shared lib classloader.
 * <p>
 * Inputs : {@link Jbi}, component install root URI <br/>
 * The task has got a non-robust approach : it stops at the first subtask failed<br />
 * and have the undo capability.
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 * @author Frederic Gardes - EBM WebSourcing
 */
public class CreateSLClassloaderTask extends AbstractLoggableTask {

    /**
     * The Petals Container
     */
    protected ContainerService containerService;

    public CreateSLClassloaderTask(final LoggingUtil log, final ContainerService containerService,
            final ContainerConfiguration containerConfiguration) {
        super(log, containerConfiguration);
        this.containerService = containerService;
    }

    public void execute(Context context) throws Exception {
        this.log.start();

        Jbi descriptor = context.getDescriptor();
        URL installUrl = context.getInstallUrl();
        SharedLibrary sharedLibraryDescriptor = descriptor.getSharedLibrary();

        // Create shared lib classloader
        final SharedLibraryLifeCycle sharedLibraryLifeCycle = this.containerService
                .createSharedLibraryLifeCycle(sharedLibraryDescriptor);

        // Start the new installer
        this.containerService.startSharedLibraryLifeCycle(sharedLibraryDescriptor.getIdentification().getName(),
                sharedLibraryDescriptor.getVersion());

        boolean parentFirst = ClassLoaderDelegationType.PARENT_FIRST.equals(sharedLibraryDescriptor
                .getClassLoaderDelegation());
        sharedLibraryLifeCycle.init(sharedLibraryDescriptor, new URL[] { installUrl }, parentFirst);

        // load in PetalsShared Libraries
        sharedLibraryLifeCycle.loadSharedLibrary();

        this.log.end();
    }

    public void undo(Context context) throws Exception {
        this.log.call();
        
        final String slName = context.getEntityName();
        if (slName != null) {

            // Remove of the fractal installer component
            try {
                this.containerService.removeSharedLibraryLifeCycle(slName, context.getEntityVersion());
            } catch (PetalsException e) {
                this.log.error("The fractal shared library can't be removed or doesn't exist", e);
            }
        }
    }

}
