/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.control;

import java.util.logging.Logger;

import javax.jbi.messaging.MessageExchange;

import org.objectweb.fractal.fraclet.annotations.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.microkernel.api.communication.JMXService;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint;
import org.ow2.petals.microkernel.util.ParameterCheckHelper;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * The JMX implementation of the echange checker client.
 * TODO: The exchange checker must be implemented via the transporter. 
 * 
 * @author Christophe Hamerling - EBM WebSourcing
 */
@Component(provides = @Interface(name = "service", signature = ExchangeCheckerClient.class))
public class JMXExchangeCheckerClientImpl implements ExchangeCheckerClient {

    private final LoggingUtil log = new LoggingUtil(Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME));

    @Requires(name = "jmx")
    protected JMXService jmxService;

    /**
     * 
     */
    @Lifecycle(step = Step.START)
    public void start() {
        this.log.call();
    }

    /**
     * 
     */
    @Lifecycle(step = Step.STOP)
    public void stop() {
        this.log.call();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isExchangeWithConsumerOkayForComponent(final PetalsServiceEndpoint internalEndpoint,
            final MessageExchange exchange) {
        this.log.start();
        ParameterCheckHelper.isNullParameterWithLog(internalEndpoint, "Endpoint", this.log);
        boolean test = true;
        try {
            // Retrieve a connection to the JMX server of the container were the
            // component is deployed
            final String containerName = internalEndpoint.getLocation().getContainerName();
            final JMXClient server = this.jmxService.getJMXClient(containerName);
            try {
                // Ask the component if it is ok to accept the exchange
                test = server.getAdminServiceClient().isExchangeWithConsumerOkayForComponent(
                        internalEndpoint.getLocation().getComponentName(), internalEndpoint, exchange);
            } finally {
                this.jmxService.returnJMXClient(containerName, server);
            }
        } catch (final Exception e) {
            this.log.error("Problem in isExchangeWithConsumerOkayForComponent", e);
        }
        this.log.end();
        return test;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isExchangeWithProviderOkayForComponent(final PetalsServiceEndpoint internalEndpoint,
            final MessageExchange exchange) {
        this.log.start();
        if (internalEndpoint == null) {
            throw new IllegalArgumentException("Endpoint must be non null");
        }
        boolean test = true;
        try {
            // Retrieve a connection to the JMX server of the container were the
            // component is deployed
            final String containerName = internalEndpoint.getLocation().getContainerName();
            final JMXClient server = this.jmxService.getJMXClient(containerName);
            try {
                // Retrieve the endpoint description
                test = server.getAdminServiceClient().isExchangeWithProviderOkayForComponent(
                        internalEndpoint.getLocation().getComponentName(), internalEndpoint, exchange);
            } finally {
                this.jmxService.returnJMXClient(containerName, server);
            }
        } catch (final Exception e) {
            this.log.error("Problem in isExchangeWithProviderOkayForComponent", e);
        }
        this.log.end();
        return test;
    }

}
