/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.registry;

import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.objectweb.fractal.fraclet.annotations.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.petals.clientserverapi.jbi.messaging.registry.exception.RegistryException;
import org.ow2.petals.jbi.descriptor.original.generated.LinkType;
import org.ow2.petals.microkernel.api.implementation.exception.MutableImplException;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.EndpointAlreadyExistsException;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.RegistryListener;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.exception.DynamicTopologyNotLockedByCurrentThreadException;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.exception.DynamicTopologyNotLockedException;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint;
import org.ow2.petals.microkernel.api.server.FractalHelper;
import org.ow2.petals.topology.generated.Topology;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * A dummy registry just used to simplify the creation of the Fractal
 * architecture by loading an ADL file instead coding all. When the composite
 * {@value FractalHelper#SYSTEM_COMPOSITE} is created, this dummy registry will
 * be replaced by the expected one.
 * 
 * @author Christophe DENEUX - Linagora
 */
@Component(provides = @Interface(name = EndpointRegistry.FRACTAL_SRV_ITF_NAME, signature = EndpointRegistry.class))
public class DummyRegistryImpl implements EndpointRegistry {

    protected final LoggingUtil log = new LoggingUtil(Logger.getLogger(EndpointRegistry.COMPONENT_LOGGER_NAME));

    @Lifecycle(step = Step.START)
    public void start() {
        this.log.call();
    }

    @Lifecycle(step = Step.STOP)
    public void stop() {
        this.log.call();
    }

    /**
     * {@inheritDoc}
     */
    public List<Map<String, Object>> getAllInternalEndpoints() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public List<Map<String, Object>> getAllExternalEndpoints() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public List<Map<String, Object>> getAllEndpoints() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public String getDescription(final String serviceName, final String endpointName) throws RegistryException {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public void synchronizeRegistry() throws Exception {
        // NOP
    }

    /**
     * {@inheritDoc}
     */
    public List<RegistryListener> getListeners() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public List<PetalsServiceEndpoint> getInternalEndpoints() throws RegistryException {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public List<PetalsServiceEndpoint> getExternalEndpoints() throws RegistryException {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public List<PetalsServiceEndpoint> getEndpoints() throws RegistryException {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public PetalsServiceEndpoint activateEndpoint(final QName serviceName, final String endpointName,
            final List<QName> interfaces, final Document description, final PetalsServiceEndpoint address,
            final boolean canOverwrite) throws EndpointAlreadyExistsException, RegistryException {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public void deactivateEndpoint(String endpointName, QName serviceName) throws RegistryException {
        // NOP
    }

    /**
     * {@inheritDoc}
     */
    public void registerExternalEndpoint(javax.jbi.servicedesc.ServiceEndpoint serviceEndpoint)
            throws RegistryException {
        // NOP
    }

    /**
     * {@inheritDoc}
     */
    public void deregisterExternalEndpoint(String endpointName, QName serviceName) throws RegistryException {
        // NOP
    }

    /**
     * {@inheritDoc}
     */
    public void registerConnection(QName consInterface, QName provService, String provEndpoint)
            throws RegistryException {
        // NOP
    }

    /**
     * {@inheritDoc}
     */
    public void registerConnection(QName consService, String consEndpoint, QName provService,
            String provEndpoint)
            throws RegistryException {
        // NOP
    }

    /**
     * {@inheritDoc}
     */
    public void deregisterConnection(QName consInterface, QName provService, String provEndpoint)
            throws RegistryException {
        // NOP
    }

    /**
     * {@inheritDoc}
     */
    public void deregisterConnection(QName consService, String consEndpoint, QName provService,
            String provEndpoint)
            throws RegistryException {
        // NOP
    }

    /**
     * {@inheritDoc}
     */
    public Document getEndpointDescriptorForEndpoint(PetalsServiceEndpoint endpoint) throws RegistryException {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public QName[] getInterfacesForEndpoint(PetalsServiceEndpoint endpoint) {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public PetalsServiceEndpoint[] getInternalEndpointsForInterface(QName interfaceName, LinkType linktype)
            throws RegistryException {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public PetalsServiceEndpoint[] getInternalEndpointsForService(QName serviceName, LinkType linktype)
            throws RegistryException {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public PetalsServiceEndpoint[] getExternalEndpointsForInterface(QName interfaceName) throws RegistryException {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public PetalsServiceEndpoint[] getExternalEndpointsForService(QName serviceName) throws RegistryException {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public PetalsServiceEndpoint getEndpoint(QName service, String name) throws RegistryException {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public void removeAllLocalEndpoints() throws RegistryException {
        // NOP
    }

    /**
     * {@inheritDoc}
     */
    public boolean lockDynamicTopology() throws InterruptedException {
        return false;
    }

    /**
     * {@inheritDoc}
     */
    public void unlockDynamicTopology() throws DynamicTopologyNotLockedByCurrentThreadException {
        // NOP

    }

    /**
     * {@inheritDoc}
     */
    public void setDynamicTopology(final Topology topology) throws DynamicTopologyNotLockedException,
            DynamicTopologyNotLockedByCurrentThreadException {
        // NOP

    }

    /**
     * {@inheritDoc}
     */
    public Topology getDynamicTopology() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public Properties getConfiguration(final String providedTopologyPassphrase) {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public Document getTopologyPartConfiguration(final Properties properties) throws MutableImplException {
        return null;
    }
}
