/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.server;

import java.util.Properties;

import org.ow2.petals.clientserverapi.server.exception.PetalsCompositeCtrlException;
import org.ow2.petals.microkernel.api.server.FractalHelper;
import org.ow2.petals.microkernel.api.system.SystemStateService;

/**
 * <p>
 * Controller of the Fractal composite 'Petals'.
 * </p>
 * <p>
 * The Fractal composite 'Petals' is in charge of loading all Fractal architecture. The controller will load all the
 * Fractal architecture except the component 'Configuration' according to the configuration.
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public interface PetalsCompositeController {

    /**
     * Logger name
     */
    public static final String COMPONENT_LOGGER_NAME = "Petals.Server";

    /**
     * Name of the server interface of the Fractal component {@value FractalHelper#PETALS_COMPOSITE_CTRL_COMPONENT}
     */
    public static final String FRACTAL_SRV_ITF_NAME = "service";

    /**
     * Name of the client interface 'configuration' of the composite 'Communication'
     */
    public static final String COMMUNICATION_CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * Name of the client interface 'registry' of the composite 'Communication'
     */
    public static final String COMMUNICATION_REGISTRY_FRACTAL_ITF_NAME = "registry";

    /**
     * Name of the client interface 'petals-composite-controller' of the composite 'Communication'
     */
    public static final String COMMUNICATION_PETALS_COMPOSITE_CTRL_FRACTAL_ITF_NAME = "petalsCompositeController";

    /**
     * Name of the server interface 'topology' of the composite 'Communication'
     */
    public static final String COMMUNICATION_TOPOLOGY_FRACTAL_SRV_ITF_NAME = "topology";

    /**
     * Name of the server interface 'jmx' of the composite 'Communication'
     */
    public static final String COMMUNICATION_JMX_FRACTAL_SRV_ITF_NAME = "jmx";

    /**
     * Name of the server interface 'jndi' of the composite 'Communication'
     */
    public static final String COMMUNICATION_JNDI_FRACTAL_SRV_ITF_NAME = "jndi";

    /**
     * Name of the client interface 'configuration' of the composite 'System'
     */
    public static final String SYSTEM_CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * Name of the <b>internal</b> client interface 'configuration' of the composite 'System'
     */
    public static final String SYSTEM_CONFIGURATION_FRACTAL_INT_ITF_NAME = "configuration";

    /**
     * Name of the client interface 'topology' of the composite 'System'
     */
    public static final String SYSTEM_TOPOLOGY_FRACTAL_ITF_NAME = "topology";

    /**
     * Name of the <b>internal</b> client interface 'topology' of the composite 'System'
     */
    public static final String SYSTEM_TOPOLOGY_FRACTAL_INT_ITF_NAME = "topology";

    /**
     * Name of the server interface 'systemstate' of the composite 'System'
     */
    public static final String SYSTEM_SYSTEMSTATE_FRACTAL_SRV_ITF_NAME = "systemState";

    /**
     * Name of the server interface 'registry' of the composite 'System'
     */
    public static final String SYSTEM_REGISTRY_FRACTAL_SRV_ITF_NAME = "registry";

    /**
     * Name of the internal server interface 'registry' of the composite 'System'
     */
    public static final String SYSTEM_REGISTRY_FRACTAL_INT_SRV_ITF_NAME = "registry";

    /**
     * Name of the server interface 'classloader' of the composite 'System'
     */
    public static final String SYSTEM_CLASSLOADER_FRACTAL_SRV_ITF_NAME = "classloader";

    /**
     * Name of the server interface 'repository' of the composite 'System'
     */
    public static final String SYSTEM_REPOSITORY_FRACTAL_SRV_ITF_NAME = "repository";

    /**
     * Name of the server interface 'persistence' of the composite 'System'
     */
    public static final String SYSTEM_PERSISTENCE_FRACTAL_SRV_ITF_NAME = "persistence";

    /**
     * Name of the client interface 'configuration' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * Name of the client interface 'jmx' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_JMX_FRACTAL_ITF_NAME = "jmx";

    /**
     * Name of the client interface 'topology' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_TOPOLOGY_FRACTAL_ITF_NAME = "topology";

    /**
     * Name of the client interface 'systemstate' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_SYSTEMSTATE_FRACTAL_ITF_NAME = "systemstate";

    /**
     * Name of the client interface 'deployment' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_DEPLOYMENT_FRACTAL_ITF_NAME = "deployment";

    /**
     * Name of the client interface 'atomicDeployment' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_ATOMICDEPLOYMENT_FRACTAL_ITF_NAME = "atomicDeployment";

    /**
     * Name of the client interface 'installation' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_INSTALLATION_FRACTAL_ITF_NAME = "installation";

    /**
     * Name of the client interface 'componentInstallation' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_COMPONENTINSTALLATION_FRACTAL_ITF_NAME = "componentInstallation";

    /**
     * Name of the client interface 'registry' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_REGISTRY_FRACTAL_ITF_NAME = "registry";

    /**
     * Name of the client interface 'admin' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_ADMIN_FRACTAL_ITF_NAME = "admin";

    /**
     * Name of the client interface 'containerService' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_CONTAINERSERVICE_FRACTAL_ITF_NAME = "containerService";

    /**
     * Name of the client interface 'petalsAdmin' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_PETALSADMIN_FRACTAL_ITF_NAME = "petalsAdmin";

    /**
     * Name of the server interface 'installationExtensionsManager' of the composite 'ExtensionsManager'
     */
    public static final String POSTEXTENSIONSMANAGER_INSTALLATIONEXTENSIONSMANAGER_FRACTAL_SRV_ITF_NAME = "installationExtensionsManager";

    /**
     * Name of the server interface 'deploymentService' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_DEPLOYMENT_FRACTAL_SRV_ITF_NAME = "deploymentService";

    /**
     * Name of the server interface 'atomicDeploymentService' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_ATOMICDEPLOYMENT_FRACTAL_SRV_ITF_NAME = "atomicDeploymentService";

    /**
     * Name of the server interface 'installationService' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_INSTALLATION_FRACTAL_SRV_ITF_NAME = "installationService";

    /**
     * Name of the server interface 'componentInstallationService' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_COMPONENTINSTALLATION_FRACTAL_SRV_ITF_NAME = "componentInstallationService";

    /**
     * Name of the server interface 'adminService' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_ADMIN_FRACTAL_SRV_ITF_NAME = "adminService";

    /**
     * Name of the server interface 'containerService' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_CONTAINER_FRACTAL_SRV_ITF_NAME = "containerService";

    /**
     * Name of the client interface 'configuration' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * Name of the client interface 'systemstate' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_SYSTEMSTATE_FRACTAL_ITF_NAME = "systemstate";

    /**
     * Name of the client interface 'jmx' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_JMX_FRACTAL_ITF_NAME = "jmx";

    /**
     * Name of the client interface 'classloader' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_CLASSLOADER_FRACTAL_ITF_NAME = "classloader";

    /**
     * Name of the client interface 'repository' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_REPOSITORY_FRACTAL_ITF_NAME = "repository";

    /**
     * Name of the client interface 'registry' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_REGISTRY_FRACTAL_ITF_NAME = "endpoint";

    /**
     * Name of the client interface 'router' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_ROUTER_FRACTAL_ITF_NAME = "router";

    /**
     * Name of the client interface 'installationExtensionManager' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_INSTALLATIONEXTENSIONSMANAGER_FRACTAL_ITF_NAME = "installationExtensionManager";

    /**
     * Name of the client interface 'jndi' of the composite 'JBI-Management'
     */
    public static final String JBIMANAGEMENT_JNDI_FRACTAL_ITF_NAME = "jndi";

    /**
     * Name of the server interface 'router' of the composite 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_ROUTER_FRACTAL_SRV_ITF_NAME = "router";

    /**
     * Name of the server interface 'transportlistener' of the composite 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_TRANSPORTLISTENER_FRACTAL_SRV_ITF_NAME = "transportlistener";

    /**
     * Name of the client interface 'configuration' of the component 'PetalsAdminService'
     */
    public static final String PETALSADMIN_CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * Name of the client interface 'topology' of the component 'PetalsAdminService'
     */
    public static final String PETALSADMIN_TOPOLOGY_FRACTAL_ITF_NAME = "topology";

    /**
     * Name of the client interface 'deployment' of the component 'PetalsAdminService'
     */
    public static final String PETALSADMIN_DEPLOYMENT_FRACTAL_ITF_NAME = "deployment";

    /**
     * Name of the client interface 'installation' of the component 'PetalsAdminService'
     */
    public static final String PETALSADMIN_INSTALLATION_FRACTAL_ITF_NAME = "installation";

    /**
     * Name of the client interface 'registry' of the component 'PetalsAdminService'
     */
    public static final String PETALSADMIN_REGISTRY_FRACTAL_ITF_NAME = "endpoint";

    /**
     * Name of the client interface 'configuration' of the component 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * Name of the client interface 'topology' of the component 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_TOPOLOGY_FRACTAL_ITF_NAME = "topology";

    /**
     * Name of the client interface 'jndi' of the component 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_JNDI_FRACTAL_ITF_NAME = "jndi";

    /**
     * Name of the client interface 'jmx' of the component 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_JMX_FRACTAL_ITF_NAME = "jmx";

    /**
     * Name of the client interface 'persistence' of the component 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_PERSISTENCE_FRACTAL_ITF_NAME = "persistence";

    /**
     * Name of the client interface 'transporter-local' of the component 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_TRANSPORTERLOCAL_FRACTAL_ITF_NAME = "transporter-local";

    /**
     * Name of the client interface 'transporter-tcp' of the component 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_TRANSPORTERTCP_FRACTAL_ITF_NAME = "transporter-tcp";

    /**
     * Name of the client interface 'endpoint' of the component 'JBI-Messaging'
     */
    public static final String JBIMESSAGING_REGISTRY_FRACTAL_ITF_NAME = "endpoint";

    /**
     * Name of the client interface 'configuration' of the component 'Transporter'
     */
    public static final String TRANSPORTER_CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * Name of the client interface 'topology' of the component 'Transporter'
     */
    public static final String TRANSPORTER_TOPOLOGY_FRACTAL_ITF_NAME = "topology";

    /**
     * Name of the client interface 'transportlistener' of the component 'Transporter'
     */
    public static final String TRANSPORTER_TRANSPORTLISTENER_FRACTAL_ITF_NAME = "transportlistener";

    /**
     * Name of the server interface 'transporter-local' of the composite 'Transporter'
     */
    public static final String TRANSPORTER_TRANSPORTERLOCAL_FRACTAL_SRV_ITF_NAME = "transporter-local";

    /**
     * Name of the server interface 'transporter-tcp' of the composite 'Transporter'
     */
    public static final String TRANSPORTER_TRANSPORTERTCP_FRACTAL_SRV_ITF_NAME = "transporter-tcp";

    /**
     * Name of the client interface 'configuration' of the component 'Container'
     */
    public static final String CONTAINER_CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * Name of the client interface 'systemstate' of the component 'Container'
     */
    public static final String CONTAINER_SYSTEMSTATE_FRACTAL_ITF_NAME = "systemstate";

    /**
     * Name of the client interface 'classloader' of the composite 'Container'
     */
    public static final String CONTAINER_CLASSLOADER_FRACTAL_ITF_NAME = "classloader";

    /**
     * Name of the client interface 'repository' of the composite 'Container'
     */
    public static final String CONTAINER_REPOSITORY_FRACTAL_ITF_NAME = "repository";

    /**
     * Name of the client interface 'registry' of the composite 'Container'
     */
    public static final String CONTAINER_REGISTRY_FRACTAL_ITF_NAME = "endpoint";

    /**
     * Name of the client interface 'router' of the composite 'Container'
     */
    public static final String CONTAINER_ROUTER_FRACTAL_ITF_NAME = "router";

    /**
     * Name of the client interface 'admin' of the composite 'Container'
     */
    public static final String CONTAINER_ADMIN_FRACTAL_ITF_NAME = "admin";

    /**
     * Stop and shutdown all the Fractal components associated to the JBI components and SAs
     * 
     * @throws PetalsCompositeCtrlException
     *             An error occurs
     */
    public void stopJbiArtefacts() throws PetalsCompositeCtrlException;

    /**
     * <p>
     * Stop the Fractal sub-system processing message exchanges:
     * <ul>
     * <li>the composite {@link FractalHelper#JBI_MESSAGING_COMPOSITE},</li>
     * <li>and the composite {@link FractalHelper#TRANSPORTER_COMPOSITE}.</li>
     * </ul>
     * </p>
     * <p>
     * No more message will be sent. Pending exchanges are ended before to stop router and transporters.
     * </p>
     * 
     * @throws PetalsCompositeCtrlException
     *             An error occurs stopping the messaging sub-system
     */
    public void stopMessagingSubsystem() throws PetalsCompositeCtrlException;

    /**
     * <p>
     * Start the Fractal sub-system processing message exchanges:
     * <ul>
     * <li>the composite {@link FractalHelper#JBI_MESSAGING_COMPOSITE},</li>
     * <li>and the composite {@link FractalHelper#TRANSPORTER_COMPOSITE}.</li>
     * </ul>
     * </p>
     * 
     * @throws PetalsCompositeCtrlException
     *             An error occurs stopping the messaging sub-system
     */
    public void startMessagingSubsystem() throws PetalsCompositeCtrlException;

    /**
     * <p>
     * Stop the Fractal component {@link FractalHelper#REGISTRY_COMPONENT}.
     * </p>
     * 
     * @throws PetalsCompositeCtrlException
     *             An error occurs stopping the registry
     */
    public void stopRegistryComponent() throws PetalsCompositeCtrlException;

    /**
     * <p>
     * Start the Fractal component {@link FractalHelper#REGISTRY_COMPONENT}.
     * </p>
     * 
     * @throws PetalsCompositeCtrlException
     *             An error occurs stopping the registry
     */
    public void startRegistryComponent() throws PetalsCompositeCtrlException;

    /**
     * Retrieve the registry implementation of the sub-domain of the current container
     * 
     * @return The registry implementation of the sub-domain
     */
    public String getRegistryImplementation();

    /**
     * <p>
     * Retrieve the configuration of the <b>registry</b> mutable implementation as {@link Properties}. See API of the
     * registry mutable implementation for the property list.
     * </p>
     * <p>
     * All properties are prefixed by the class name of the registry mutable implementation
     * </p>
     * 
     * @param providedTopologyPassphrase
     *            Passphrase needed to return sensible information. If <code>null</code>, empty or not matching the one
     *            of the container on which the connection is established, no sensible information will be returned.
     * @return The configuration of the registry mutable implementation as {@link Properties}. Not <code>null</code>.
     * @throws PetalsCompositeCtrlException
     *             An error occurs getting the configuration of the registry mutable implementation
     */
    public Properties getRegistryConfiguration(final String providedTopologyPassphrase)
            throws PetalsCompositeCtrlException;

    /**
     * <p>
     * Stop the Fractal component {@link FractalHelper#SYSTEMRECOVERY_COMPONENT}.
     * </p>
     * 
     * @throws PetalsCompositeCtrlException
     *             An error occurs stopping the system recovery service
     */
    public void stopSystemRecoveryService() throws PetalsCompositeCtrlException;

    /**
     * <p>
     * Start the Fractal component {@link FractalHelper#SYSTEMRECOVERY_COMPONENT}.
     * </p>
     * 
     * @throws PetalsCompositeCtrlException
     *             An error occurs stopping the system recovery service
     */
    public void startSystemRecoveryService() throws PetalsCompositeCtrlException;

    /**
     * Recover the system (JBI artifacts previously deployed) from the {@link SystemStateService}. If needed, the
     * recovering system service is started.
     * 
     * @throws PetalsCompositeCtrlException
     *             An error occurs recovering the system
     */
    public void recoverSystem() throws PetalsCompositeCtrlException;

    /**
     * Remove from the Fractal framework all components associated to a JBI artifact. They are not undeployed or
     * uninstalled, only freed from memory.
     * 
     * @throws PetalsCompositeCtrlException
     *             An error occurs freeing all JBI artifacts
     */
    public void freeAllJbiArtifacts() throws PetalsCompositeCtrlException;

    /**
     * The configuration of the Petals container has changed and must be taken into account.
     * 
     * @throws PetalsCompositeCtrlException
     */
    public void onConfigurationUpdated() throws PetalsCompositeCtrlException;

}
